#ifndef	PCPTALK_H
#define	PCPTALK_H
/*
 * Written by Oron Peled <oron@actcom.co.il>
 * Copyright (C) 2009, Xorcom
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * PCPTALK - Panel-Control-Protocol:
 *	     A dialect of XTALK to manage Xorcom control panels.
 */

#include <stdint.h>
#include <xtalk/proto_sync.h>
#include "xpanel_visibility.h"

struct pcp_device;

#define	PCP_MAX_KEYBUF	10	/* Maximum number of keys returned */
#define	PCP_BACKLIGHT_TO 300	/*Timeout for the panel backlight switching off (sec) */
#define	PCP_MAX_FANS	3	/* Number of fans */
#define PCP_PSU_STATUS_ON	0x02  /* Both PSU are on */
#define PCP_PSU_STATUS_OFF	0x01  /* One of PSU is off */
#define PCP_PSU_STATUS_DISCONNECTED	0x03 /* Both PSU are disconnected */ 

/*
 * Key codes for the panel keyboard
 */
enum keycodes {
	KEYCODE_UNUSED = 0,	/* Don't use 0, just in case... */
	/* Numerals */
	KEYCODE_0,
	KEYCODE_1,
	KEYCODE_2,
	KEYCODE_3,
	KEYCODE_4,
	KEYCODE_5,
	KEYCODE_6,
	KEYCODE_7,
	KEYCODE_8,
	KEYCODE_9,
	/* Editing */
	KEYCODE_DEL,
	KEYCODE_STAR,
	KEYCODE_ESC,
	KEYCODE_ENTER,
	KEYCODE_HOME,
	/* Directions */
	KEYCODE_SOUTH,
	KEYCODE_NORTH,
	KEYCODE_WEST,
	KEYCODE_EAST,
	/* Special Functions */
	KEYCODE_BAK,	/* Backup		*/
	KEYCODE_REC,	/* Recover		*/
	KEYCODE_MENU,	/* Menu			*/
	KEYCODE_TUNNEL,	/* Support tunnel	*/
	KEYCODE_DOWN,	/* Download		*/
	KEYCODE_IPADDR,	/* Set IP-Address	*/

	KEYCODE_UNUSED_LAST	/* A range marker -- must be last */
};

/* API */
XPANEL_API	struct pcp_device *pcp_new(struct xusb_iface *iface);
XPANEL_API	void pcp_delete(struct pcp_device *pcpdev);
XPANEL_API	uint8_t pcp_get_panel_proto_version(struct pcp_device *pcpdev);

XPANEL_API	int pcp_lcd_sendd(struct pcp_device *pcpdev, char ascii);

/* LCD display operations */
/* D0 */ XPANEL_API	int pcp_lcd_clear_display(struct pcp_device *pcpdev);
/* D1 */ XPANEL_API	int pcp_lcd_cursor_home(struct pcp_device *pcpdev);
/* D2 */ XPANEL_API	int pcp_lcd_entry_mode(struct pcp_device *pcpdev, int i_d, int s);
/* D3 */ XPANEL_API	int pcp_lcd_display_control(struct pcp_device *pcpdev, int d, int c, int b);
/* D4 */ XPANEL_API	int pcp_lcd_display_shift(struct pcp_device *pcpdev, int s_c, int r_l);
/* D5 */ XPANEL_API	int pcp_lcd_function_set(struct pcp_device *pcpdev, int d_l, int n, int f);
/* D7 */ XPANEL_API	int pcp_lcd_gotoxy(struct pcp_device *pcpdev, uint8_t x, uint8_t y);

XPANEL_API	int pcp_backlight(struct pcp_device *pcpdev, int on);
XPANEL_API	int pcp_getbacklightstatus(struct pcp_device *pcpdev);
XPANEL_API	int pcp_buzzer(struct pcp_device *pcpdev, int on);

XPANEL_API	int pcp_led(struct pcp_device *pcpdev, uint8_t bitmask);
XPANEL_API	int pcp_keyget(struct pcp_device *pcpdev, uint8_t keybuf[]);
XPANEL_API	int pcp_reset(struct pcp_device *pcpdev);
XPANEL_API	int pcp_fanspeed_set(struct pcp_device *pcpdev, uint8_t fan_number, uint8_t fan_speed);
XPANEL_API	int pcp_fanspeed_read(struct pcp_device *pcpdev, uint8_t fan_number);
XPANEL_API	int pcp_fanspeed(struct pcp_device *pcpdev, uint8_t fan_number);	/* cached */
XPANEL_API	int pcp_fanspeed_change(struct pcp_device *pcpdev, uint8_t fan_number, int delta);
XPANEL_API	int pcp_psu_read(struct pcp_device *pcpdev, int *psu_sts);
XPANEL_API	int pcp_zigbee_lamp(struct pcp_device *pcpdev, uint8_t lamp_num, int on);
XPANEL_API	const char *keycode_name(uint8_t keycode);

#endif	/* PCPTALK_H */
