#ifndef	XPW_H
#define	XPW_H

/*
 * Xorcom Panel Widgets -- XPW
 */

#include <xpanel/pcptalk.h>
#include <xpanel/pcpscreen.h>
#include <xpanel/xpanel_visibility.h>

/* Forward declaractions */
enum classid;
struct xpw_class;
struct xpw;
struct xpw_modal;

/*
 * A Key handler:
 * 	Receives: the current widget and the keycode pressed.
 *	Returns: The next widget (may be the same)
 */
typedef void (*widget_kbd_handler_t)(struct xpw *widget, enum keycodes key);
typedef void (*widget_notify_t)(struct xpw *widget, enum keycodes key, void *data);
typedef void (*widget_func_t)(struct xpw *widget);

#define	WIDGET(p)	((struct xpw *)(p))
#define WIDGET_NAME_MAX_LEN	20
/*
 * The toplevel widget (normally xpw_menu) bellow the
 * background widget.
 */
XPANEL_API	struct xpw	*xpw_toplevel(void);


enum xpw_classid {
	XPW_UNUSED = 0,	/* Must be 0 */
	XPW_BACKGROUND = 1,
	XPW_MODAL,
	XPW_DIALOG,
	XPW_MENU_NOTIFIER,
	XPW_MENU,
	XPW_EDITBOX,
	XPW_IPADDR,
	XPW_YESNO,
	XPW_DISPLAYBOX,
	XPW_PASSWD,
	/* ... etc... */
	XPW_CLASS_LAST		/* Must be last */
};

struct xpw_class {
	enum xpw_classid	classid;
	const char		*name;
	widget_kbd_handler_t	kbd_handlers[KEYCODE_UNUSED_LAST];
	void	(*dump_widget)(struct xpw *widget, int depth, const char *prefix);
	void	(*redraw)(struct xpw_screen *screen, struct xpw *widget);
	void	(*restart)(struct xpw *widget);
};

/*
 * Convenience wrappers
 */
XPANEL_API	void	widget_redraw(struct xpw_screen *screen, struct xpw *widget);
XPANEL_API	void	widget_restart(struct xpw *widget);
XPANEL_API	void	widget_notify(struct xpw *widget, enum keycodes key);
XPANEL_API	void	widget_call_handler(struct xpw *widget, enum keycodes key);

struct xpw {
	enum xpw_classid	classid;
	char			name[WIDGET_NAME_MAX_LEN+1];
	widget_notify_t		notify;
	void			*user_data;
	struct xpw_modal	*modal;
	/* For dynamic widgets */
	widget_func_t	populate;
	widget_func_t	cleanup;
	/* For hotkey masking */
	uint64_t		hotkeys_masked;
};

XPANEL_API	int  xpw_construct(struct xpw *xpw, enum xpw_classid classid, const char *name);
XPANEL_API	int  xpw_set_callback(struct xpw *xpw, widget_notify_t notify, void *data);
XPANEL_API	int  xpw_set_dynamic(struct xpw *xpw, widget_func_t populate, widget_func_t cleanup);
XPANEL_API	int  xpw_set_modal(struct xpw *xpw, struct xpw_modal *modal, struct xpw *delegate);
XPANEL_API	void xpw_hotkey_set_mask(struct xpw *xpw, const uint64_t *new_mask, uint64_t *old_mask);
XPANEL_API	int  xpw_hotkey_is_masked(struct xpw *xpw, enum keycodes key);

void handle_modal(struct xpw *widget);

XPANEL_API	int xpw_init(struct xpw *toplevel);
XPANEL_API	int xpw_fini(void);

XPANEL_PRIVATE	int xpw_class_register(const struct xpw_class *xpw_class);
XPANEL_PRIVATE	int xpw_class_unregister(enum xpw_classid classid);

XPANEL_API	const struct xpw_class *xpw_getclass(enum xpw_classid classid);
XPANEL_API	const struct xpw_class *xpw_myclass(struct xpw *xpw);
XPANEL_API	const char *xpw_myname(struct xpw *xpw);

/*
 * Widget stack handling
 */
XPANEL_API	int xpw_stack_depth(void);
XPANEL_API	int xpw_stack_push(struct xpw *xpw);
XPANEL_API	int xpw_stack_pop(void);
XPANEL_API	struct xpw *xpw_stack_top(void);
XPANEL_API	struct xpw *xpw_current(void);
XPANEL_API	struct xpw *xpw_stack_peek(unsigned int depth);
XPANEL_API	int xpw_stack_string(char str[], int maxlen);
XPANEL_API	void dump_xpw_stack(const char *msg);

XPANEL_API	struct xpw *process_key(struct xpw *widget, enum keycodes key);
XPANEL_API	void dump_prefix(int depth, const char *prefix);

#endif	/* XPW_H */
