#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <xtalk/debug.h>
#include <xpanel/pcptalk.h>
#include <xpanel/pcpscreen.h>

#define	DBG_MASK	0x80
#define	MAX_X	16

struct xpw_screen {
	struct pcp_device	*pcpdev;
	int			x;
	int			y;
	int			nesting;
	int			keybeep_yes;
};

struct xpw_screen *xpw_screen_new(void *priv)
{
	struct xpw_screen *screen;

	if((screen = malloc(sizeof(*screen))) == NULL) {
		ERR("Out of memory\n");
		return NULL;
	}
	memset(screen, 0, sizeof(*screen));
	screen->pcpdev = (struct pcp_device *)priv;
	screen->x = 0;
	screen->y = 0;
	screen->nesting = 0;
	screen->keybeep_yes = 1;
	return screen;
}

void xpw_screen_destroy(struct xpw_screen *screen)
{
	if(screen) {
		memset(screen, 0, sizeof(*screen));
		free(screen);
	}
}

struct pcp_device *xpw_screen_getphys(const struct xpw_screen *screen)
{
	return screen->pcpdev;
}

void xpw_screen_nesting(struct xpw_screen *screen, int nesting)
{
	int	y;
	int	x;

	if(!nesting)
		goto out;
	DBG("%s:\n", __func__);
	screen->nesting = 0;	/* temporarily */
	for(y = 0; y <= 1; y++) {
		xpw_screen_gotoxy(screen, 0, y);
		for(x = 0; x < MAX_X && x < nesting; x++) {
			pcp_lcd_sendd(screen->pcpdev, '>');
		}
	}
out:
	xpw_screen_gotoxy(screen, screen->x, screen->y);
	screen->nesting = nesting;
}

void xpw_screen_clear(struct xpw_screen *screen)
{
	pcp_lcd_clear_display(screen->pcpdev);
	screen->x = 0;
	screen->y = 0;
}

void xpw_screen_where(struct xpw_screen *screen, int *x, int *y)
{
	*x = screen->x;
	*y = screen->y;
}

void xpw_screen_gotoxy(struct xpw_screen *screen, int x, int y)
{
	DBG("%s: (%d) %d,%d\n", __func__, screen->nesting, x, y);
	if(pcp_lcd_gotoxy(screen->pcpdev, screen->nesting + x, y) >= 0) {
		screen->x = x;
		screen->y = y;
	}
}

void xpw_screen_string(struct xpw_screen *screen, const char *str)
{
	int	i;
	int	endx;

	for(i = 0; *(str + i); i++) {
		pcp_lcd_sendd(screen->pcpdev, *(str + i));
		screen->x++;
	}
	endx = screen->x;
	DBG("Text to %d: %s\n", screen->x, str);
	for(i = screen->x; i < MAX_X; i++) {
		pcp_lcd_sendd(screen->pcpdev, ' ');
		screen->x++;
	}
	xpw_screen_gotoxy(screen, endx, screen->y);
}

void xpw_screen_beep(struct xpw_screen *screen, int usec)
{
	pcp_buzzer(screen->pcpdev, 1);
	usleep(usec);
	pcp_buzzer(screen->pcpdev, 0);
}
