#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <arpa/inet.h>
#include <xtalk/debug.h>
#include <xpanel/xpw.h>
#include <xpanel/xpw_displaybox.h>
#include <xpanel/xpw_hotkeys.h>

#define	DBG_MASK	0x80

static void dump_displaybox(struct xpw *widget, int depth, const char *prefix)
{
	struct xpw_displaybox	*dbox;
	char	*p;
	int		i;
	int		inside = 0;

	assert(widget->classid == XPW_DISPLAYBOX);
	dbox = (struct xpw_displaybox *)widget;
	dump_prefix(depth, prefix);
	printf("\"%s\":\n", widget->name);
	depth++;
	dump_prefix(depth, "[");
	p = dbox->contents;
	for(i = 0; i < XPW_DISPLAYBOX_MAXWIDTH && *p; i++, p++) {
		if(i == dbox->cursor_pos) {
			inside = 1;
			putchar('|');
		}
		putchar(*p);
		if(i == dbox->cursor_pos)
			putchar('|');
	}
	if(!inside)
		printf("|.|");
	printf("]\n");
}

static void redraw_displaybox(struct xpw_screen *screen, struct xpw *widget)
{
	struct xpw_displaybox	*dbox;

	assert(widget->classid == XPW_DISPLAYBOX);
	dbox = (struct xpw_displaybox	*)widget;
	DBG("[%s]\n", dbox->contents);
	xpw_screen_gotoxy(screen, 0, 0);
	xpw_screen_string(screen, dbox->prompt);
	xpw_screen_gotoxy(screen, 0, 1);
	xpw_screen_string(screen, dbox->contents);
	xpw_screen_gotoxy(screen, dbox->cursor_pos, 1);
}

static void restart_displaybox(struct xpw *widget)
{
  return;
}

static void displaybox_handler(struct xpw *widget, enum keycodes key)
{
	struct xpw		*next_widget;

	//dump_xpw_stack("displaybox_handler start");
	next_widget = NULL;
	switch(key) {
		case KEYCODE_ESC:
			if(xpw_stack_pop() == -ENOENT) {
				WARN("Hitting top. Staying there.\n");
			}
			next_widget = xpw_current();
			break;
		default:
			break;
	}
	if(!next_widget)
		next_widget = widget;

	DBG("displaybox_handler: \"%s\" -> \"%s\"\n", widget->name, next_widget->name);
}

static struct xpw_class	xpw_displaybox_class = {
	.classid	= XPW_DISPLAYBOX,
	.name		= "DisplayBox",
	.kbd_handlers = {
		[KEYCODE_0]	displaybox_handler,
		[KEYCODE_1]	displaybox_handler,
		[KEYCODE_2]	displaybox_handler,
		[KEYCODE_3]	displaybox_handler,
		[KEYCODE_4]	displaybox_handler,
		[KEYCODE_5]	displaybox_handler,
		[KEYCODE_6]	displaybox_handler,
		[KEYCODE_7]	displaybox_handler,
		[KEYCODE_8]	displaybox_handler,
		[KEYCODE_9]	displaybox_handler,
		[KEYCODE_STAR]	displaybox_handler,
		[KEYCODE_DEL]	displaybox_handler,
		[KEYCODE_WEST]	displaybox_handler,
		[KEYCODE_EAST]	displaybox_handler,
		[KEYCODE_ESC]	displaybox_handler,
		[KEYCODE_ENTER]	displaybox_handler,
	},
	.dump_widget	= dump_displaybox,
	.redraw		= redraw_displaybox,
	.restart	= restart_displaybox,
};

int xpw_displaybox_construct(struct xpw_displaybox	*xpw_dbox, const char *prompt)
{
	xpw_construct(&xpw_dbox->myself, XPW_DISPLAYBOX, prompt);
	snprintf(xpw_dbox->prompt, sizeof(xpw_dbox->prompt), "%s:", prompt);
	memset(xpw_dbox->contents, 0, sizeof(xpw_dbox->contents));
	xpw_dbox->cursor_pos = 0;
	return 0;
}

int xpw_displaybox_set(struct xpw_displaybox *xpw_dbox, const char *text)
{
	strncpy( xpw_dbox->contents, text, XPW_DISPLAYBOX_MAXWIDTH);
	xpw_dbox->contents[ XPW_DISPLAYBOX_MAXWIDTH] = 0;
	return 0;
}

int xpw_displaybox_register(void)
{
	return xpw_class_register(&xpw_displaybox_class);
}

void xpw_displaybox_unregister(void)
{
	//xpw_hotkey_unset(KEYCODE_IPADDR);
	xpw_class_unregister(XPW_DISPLAYBOX);
}
