#include <assert.h>
#include <errno.h>
#include <string.h>
#include <xtalk/debug.h>
#include <xpanel/xpw_hotkeys.h>

#define	DBG_MASK	0x20

struct xpw_hotkey {
	enum keycodes		keycode;
	struct xpw		*widget;
	xpw_hotkey_func_t	func;
};

static struct xpw_hotkey	global_hotkeys[KEYCODE_UNUSED_LAST];

int xpw_hotkey_set(enum keycodes key, struct xpw *widget, xpw_hotkey_func_t func)
{
	if(key < 0 || key >= KEYCODE_UNUSED_LAST) {
		ERR("%s: invalid key %s\n", __FUNCTION__, keycode_name(key));
		return -EINVAL;
	}
	if(global_hotkeys[key].widget) {
		ERR("%s: hotkey %s is already set to '%s'\n",
			__FUNCTION__, keycode_name(key), widget->name);
		return -EINVAL;
	}
	global_hotkeys[key].keycode = key;
	global_hotkeys[key].widget = widget;
	global_hotkeys[key].func = func;
	return 0;
}

int xpw_hotkey_unset(enum keycodes key)
{
	if(key < 0 || key >= KEYCODE_UNUSED_LAST) {
		ERR("%s: invalid key %s\n", __FUNCTION__, keycode_name(key));
		return -EINVAL;
	}
	if(!global_hotkeys[key].widget) {
		DBG("%s: hotkey %s is not set\n",
			__FUNCTION__, keycode_name(key));
		return 0;
	}
	global_hotkeys[key].keycode = key;
	global_hotkeys[key].widget = NULL;
	global_hotkeys[key].func = NULL;
	return 0;
}

void xpw_hotkey_mask_full(uint64_t *mask)
{
	*mask = ~0L;
}

void xpw_hotkey_mask_empty(uint64_t *mask)
{
	*mask = 0L;
}

void xpw_hotkey_mask_add(uint64_t *mask, enum keycodes key)
{
	*mask |= (1 << key);
}

void xpw_hotkey_mask_del(uint64_t *mask, enum keycodes key)
{
	*mask &= ~(1 << key);
}

struct xpw *process_hotkey(struct xpw *from_widget, enum keycodes key)
{
	struct xpw		*hotkey_widget;
	struct xpw		*func_widget;
	xpw_hotkey_func_t	func;
	int			ret;

	assert(key >= 0 && key < KEYCODE_UNUSED_LAST);
	if (xpw_hotkey_is_masked(from_widget, key)) {
		DBG("%s: hotkey %s is masked\n", __FUNCTION__, keycode_name(key));
		return NULL;	/* Do nothing */
	}
	hotkey_widget = global_hotkeys[key].widget;
	func = global_hotkeys[key].func;
	func_widget = NULL;
	if(hotkey_widget == NULL && func == NULL) {
		DBG("%s: hotkey %s is not bound\n", __FUNCTION__, keycode_name(key));
		return NULL;
	}
	if(from_widget != NULL && from_widget == hotkey_widget) {
		DBG("Recursion(from %s): do nothing\n", hotkey_widget->name);
		return NULL;	/* Do nothing */
	}
	if(hotkey_widget) {		/* Set new widget */
		if((ret = xpw_stack_push(hotkey_widget)) < 0) {
			ERR("xpw_stack_push() failed (%s)\n", strerror(ret));
			return NULL;
		}
	}
	if(func) {			/* May replace new widget */
		func_widget = func(key, hotkey_widget);
	}
	return (func_widget) ? func_widget : hotkey_widget;
}

