#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <xtalk/debug.h>
#include <xpanel/xpw.h>
#include <xpanel/xpw_modal.h>

#define	DBG_MASK	0x80

static void dump_modal(struct xpw *widget, int depth, const char *prefix)
{
	struct xpw_modal	*modal;

	assert(widget->classid == XPW_MODAL);
	modal = (struct xpw_modal *)widget;
	dump_prefix(depth, prefix);
	printf("\"%s\": ==> \"%s\"\n", widget->name, modal->delegate->name);
}

static void redraw_modal(struct xpw_screen *screen, struct xpw *widget)
{
	struct xpw_modal	*modal;
	struct xpw		*delegate;
	const struct xpw_class	*delegate_class;

	assert(widget->classid == XPW_MODAL);
	modal = (struct xpw_modal *)widget;
	delegate = modal->delegate;
	delegate_class = xpw_getclass(delegate->classid);
	assert(delegate_class);
	delegate_class->redraw(screen, delegate);
}

/*
 * Trampoline: call our notify method
 */
static void catch_delegate_notification(struct xpw *widget, enum keycodes key, void *data)
{
	struct xpw		*modal_widget = data;

	assert(modal_widget);
	assert(modal_widget->classid == XPW_MODAL);
	INFO("%s(%s): \"%s\"\n", __func__, keycode_name(key), modal_widget->name);
	widget_notify(modal_widget, key);
}

int modal_construct(struct xpw_modal *modal, struct xpw *parent, struct xpw *delegate)
{
	assert(modal);
	xpw_construct(&modal->myself, XPW_MODAL, "Modal");
	assert(!delegate->notify);
	assert(!delegate->user_data);
	modal->parent = parent;
	modal->delegate = delegate;
	modal_active(modal, 0);
	return 0;
}

void modal_active(struct xpw_modal *modal, int on)
{
	assert(modal);
	modal->is_active = on;
	INFO("%s(%s): %s\n",
		__func__,
		modal->delegate->name,
		(on) ? "ON" : "OFF");
	if(on) {
		modal->delegate->notify = catch_delegate_notification;
		modal->delegate->user_data = modal;
		xpw_stack_push(modal->delegate);
	} else {
		modal->delegate->notify = NULL;
		modal->delegate->user_data = NULL;
	}
}

static void modal_handler(struct xpw *widget, enum keycodes key)
{
	struct xpw_modal	*modal = (struct xpw_modal *)widget;
	struct xpw		*delegate = modal->delegate;
	struct xpw		*next_widget;

	//dump_xpw_stack("modal_handler start");
	next_widget = NULL;
	widget_call_handler(delegate, key);
	next_widget = xpw_current();
	DBG("%s: \"%s\" -> \"%s\"\n", __func__, widget->name, next_widget->name);
}

static void restart_modal(struct xpw *widget)
{
	/* struct xpw_modal	*modal; */

	assert(widget->classid == XPW_MODAL);
	/* modal = (struct xpw_modal *)widget; */
}


static struct xpw_class	xpw_modal_class = {
	.classid	= XPW_MODAL,
	.name		= "Modal",
	.kbd_handlers = {
		[KEYCODE_0]	modal_handler,
		[KEYCODE_1]	modal_handler,
		[KEYCODE_2]	modal_handler,
		[KEYCODE_3]	modal_handler,
		[KEYCODE_4]	modal_handler,
		[KEYCODE_5]	modal_handler,
		[KEYCODE_6]	modal_handler,
		[KEYCODE_7]	modal_handler,
		[KEYCODE_8]	modal_handler,
		[KEYCODE_9]	modal_handler,
		[KEYCODE_STAR]	modal_handler,
		[KEYCODE_DEL]	modal_handler,
		[KEYCODE_WEST]	modal_handler,
		[KEYCODE_EAST]	modal_handler,
		[KEYCODE_ESC]	modal_handler,
		[KEYCODE_ENTER]	modal_handler,
	},
	.dump_widget	= dump_modal,
	.redraw		= redraw_modal,
	.restart	= restart_modal,
};

int xpw_modal_register(void)
{
	return xpw_class_register(&xpw_modal_class);
}

void xpw_modal_unregister(void)
{
	xpw_class_unregister(XPW_MODAL);
}
