#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <xtalk/debug.h>
#include <xpanel/xpw.h>
#include <xpanel/xpw_yesno.h>

#define	DBG_MASK	0x80

#define	POS_YES	0
#define	POS_NO	1
#define	IS_YES(x)	((x)->current_choice == POS_YES)

static void dump_yesno(struct xpw *widget, int depth, const char *prefix)
{
	struct xpw_yesno	*yesno;

	assert(widget->classid == XPW_YESNO);
	yesno = (struct xpw_yesno *)widget;
	dump_prefix(depth, prefix);
	printf("\"%s\":\n", widget->name);
	depth++;
	dump_prefix(depth, "");
	printf("%s: [\"%s\", \"%s\"] (%d)\n",
		yesno->prompt,
		yesno->answer[POS_YES], yesno->answer[POS_NO],
		yesno->current_choice);
}

static void redraw_yesno(struct xpw_screen *screen, struct xpw *widget)
{
	struct xpw_yesno	*yesno;
	int			i;

	assert(widget->classid == XPW_YESNO);
	yesno = (struct xpw_yesno *)widget;
	DBG("%s:\n", yesno->prompt);
	xpw_screen_gotoxy(screen, 0, 0);
	xpw_screen_string(screen, yesno->prompt);
	for(i = 0; i <= 1; i++) {
		DBG("%c[%s]\n", IS_YES(yesno) ? '+' : ' ', yesno->answer[i]);
		xpw_screen_gotoxy(screen, i*(PROMPT_MAXWIDTH/2), 1);
		xpw_screen_string(screen, yesno->answer[i]);
	}
	xpw_screen_gotoxy(screen, yesno->current_choice*(PROMPT_MAXWIDTH/2), 1);
}

static void restart_yesno(struct xpw *widget)
{
	struct xpw_yesno	*yesno;

	assert(widget->classid == XPW_YESNO);
	yesno = (struct xpw_yesno *)widget;
	yesno->current_choice = yesno->default_choice;
}

static void yesno_handler(struct xpw *widget, enum keycodes key)
{
	struct xpw_yesno	*yesno;
	struct xpw		*next_widget;
	widget_notify_t		notify;

	//dump_xpw_stack("yesno_handler start");
	assert(widget->classid == XPW_YESNO);
	notify = widget->notify;
	yesno = (struct xpw_yesno *)widget;
	next_widget = NULL;
	switch(key) {
		case KEYCODE_0:
		case KEYCODE_WEST:
			if(yesno->current_choice > 0)
				yesno->current_choice--;
			break;
		case KEYCODE_1:
		case KEYCODE_EAST:
			if(yesno->current_choice < 1)
				yesno->current_choice++;
			break;
		case KEYCODE_ENTER:
			INFO("YESNO(%s): IS %s\n", xpw_myname(WIDGET(yesno)),
				IS_YES(yesno) ? "YES" : "NO");
			if(notify) {
				notify(widget, key, widget->user_data);
			}
			/* Fall-through */
		case KEYCODE_ESC:
			if(xpw_stack_pop() == -ENOENT) {
				WARN("Hitting top. Staying there.\n");
			}
			next_widget = xpw_current();
			break;
		default:
			break;
	}
	if(!next_widget)
		next_widget = widget;
	DBG("yesno_handler: \"%s\" -> \"%s\"\n", widget->name, next_widget->name);
}

static struct xpw_class	xpw_yesno_class = {
	.classid	= XPW_YESNO,
	.name		= "YesNo",
	.kbd_handlers = {
		[KEYCODE_0]	yesno_handler,
		[KEYCODE_1]	yesno_handler,
		[KEYCODE_WEST]	yesno_handler,
		[KEYCODE_EAST]	yesno_handler,
		[KEYCODE_ESC]	yesno_handler,
		[KEYCODE_ENTER]	yesno_handler,
	},
	.dump_widget	= dump_yesno,
	.redraw		= redraw_yesno,
	.restart	= restart_yesno,
};

int xpw_yesno_construct(struct xpw_yesno *xpw_yesno,
	const char *prompt,
	const char *yes,
	const char *no,
	int default_choice)
{
	xpw_construct(&xpw_yesno->myself, XPW_YESNO, prompt);
	snprintf(xpw_yesno->prompt, sizeof(xpw_yesno->prompt), "%s:", prompt);
	snprintf(xpw_yesno->answer[POS_YES], sizeof(xpw_yesno->answer[POS_YES]), "%s:", yes);
	snprintf(xpw_yesno->answer[POS_NO], sizeof(xpw_yesno->answer[POS_NO]), "%s:", no);
	xpw_yesno->default_choice = xpw_yesno->current_choice =
		(default_choice) ? POS_YES : POS_NO;
	return 0;
}

int xpw_yesno_register(void)
{
	return xpw_class_register(&xpw_yesno_class);
}

void xpw_yesno_unregister(void)
{
	xpw_class_unregister(XPW_YESNO);
}

int xpw_yesno_result(struct xpw_yesno *yesno)
{
	assert(yesno);
	return IS_YES(yesno);
}
